function [res, gamma] = G_cal(P,A,C,Q,R,P_bar,delta)
% [res, gamma] = G_cal(P,A,C,Q,R,P_bar,delta)
%
% Function calligraphic G: (eq. (21) in paper mentioned below)
%
%   P(k+1) = G_cal(P(k)) 
%       = A P(k) A^T + Q - A P(k) (C(P(k)))^T 
%           ( C(P(k)) P(k) (C(P(k)))^T + R(P(k)) )^{-1} 
%           C(P(k)) P(k) A^T
% 
% (all checks dropped to simplify notation).
%
% Matrices C(P(k)) and R(P(k)) are constructed according to the number of 
% available measurements, that is, according the eq. (18)-(20).
% The decision rule (17), for using a measurement y_j is:
%
%   use y_j   iff   C_j (P(k)-P_bar) C_j^T >= \delta_j P_{y_j}.
% 
% C(P(k)) is then constructed by stacking all rows that correspond to used
% measurements; and R(P(k)) has the measurement noise variances 
% corresponding to the used measurements on its diagonal.
%
%
% --
% Supplementary material to
%
% S. Trimpe, R. D'Andrea, "Reduced Communication State Estimation for 
% Control of an Unstable Networked Control System", 50th IEEE Conference 
% on Decision and Control and European Control Conference, 2011.
%
% Copyright:
% Sebastian Trimpe
% ETH Zurich
% Institute for Dynamic Systems and Control (IDSC)
% strimpe(at)ethz.ch
% September 2011
%


%% Prep
p = size(C,1);      % total number of measurements


%% Contruct output matrices according to decision rule
% Construct matrices C(P(k)) and R(P(k)) according to the decision rule 
% (17), and equations (18)-(20).

% start with empty matrices
Ck = [];        
Rk = [];

% Check for each measurement individually.
for j=1:p
    % Check sending decision:
    gamma(j) = (C(j,:)*(P-P_bar)*(C(j,:))' >= delta(j) * (C(j,:)*P_bar*(C(j,:))'+R(j,j)));
    if (gamma(j))
        % The measurement y(j) is used.
        Ck = [Ck; C(j,:)];          % append row
        Rk = blkdiag(Rk,R(j,j));    % append diagonal element
    end;
end;


%% Compute G_cal
% Compute G_cal.

if ~isempty(Ck)
    % Measurement vector not empty; use Ck, Rk for update step.
    res = A*P*A'+Q - A*P*Ck'*inv(Ck*P*Ck'+Rk)*Ck*P*A';
else
    % Measurement vector empty; do prediction part only.
    res = A*P*A'+Q;
end;

