% simulate.m
%
% Generates simulation results presented in Fig. 5 of Sec. IV-B.
%
% Simulation of the variance iteration of reduced communication Kalman 
% filter (eq. (21)) for the design of Sec. IV-B.
%
%
% --
% Supplementary material to
%
% S. Trimpe, R. D'Andrea, "Reduced Communication State Estimation for 
% Control of an Unstable Networked Control System", 50th IEEE Conference 
% on Decision and Control and European Control Conference, 2011.
%
% Copyright:
% Sebastian Trimpe
% ETH Zurich
% Institute for Dynamic Systems and Control (IDSC)
% strimpe(at)ethz.ch
% September 2011
%


%% Configuration
% Uncomment to use latex interpreter.
set(0,'defaulttextinterpreter','latex');    % set text interpreter


%% Load Kalman filter parameters.
% See corresponding script for details.
KalmanFilterParameters;



%% Simulation parameters.
% Total number of simulation steps.
N = 1800;



%% Simulation: Iterate the variance equation.
% Iteration of the Kalman filter variance equation (21):
%
%   P(k+1) = G_cal(P(k)).
%
% For the implementation of G_cal see the corresponding file.

% Variables to hold the simulation results.
P = zeros(n,n,N);

% Initialize with solution of the DARE.
P(:,:,1) = P_bar;
for k=1:N-1
    P(:,:,k+1) = G_cal(P(:,:,k),A,C,Q,R,P_bar,delta);
end;



%% Plot the results.

% Figure 1: plot of data corresponding to Fig. 5 in paper:
%
%   \check{P}_{jj}(k)       for j=1,4,7,8 and k=1000,...,1300.
%
fig1 = figure(1); 

kk = 1000:1300;     % time range

% P_{11}: variance of module 1's angle
subplot(4,1,1);
plot(kk,squeeze(P(1,1,kk)),'k');
grid;
set(gca,'ylim',[0, 60]);
ylabel(['$\check{P}_{11}(k)$']);

% P_{44}: variance of module 4's angle
subplot(4,1,2);
plot(kk,squeeze(P(4,4,kk)),'k');
grid;
set(gca,'ylim',[0, 60]);
ylabel(['$\check{P}_{44}(k)$']);

% P_{77}: variance of cube angle
subplot(4,1,3);
plot(kk,squeeze(P(7,7,kk)),'k');
grid;
set(gca,'ylim',[0.7, 0.9]);
ylabel(['$\check{P}_{77}(k)$']);

% P_{88}: variance of cube angular velocity
subplot(4,1,4);
plot(kk,squeeze(P(8,8,kk)),'k');
grid;
set(gca,'ylim',[0.5, 6.5]);
ylabel(['$\check{P}_{88}(k)$']);
xlabel('time index $k$');

disp(' ');
disp(['The plot in Matlab Figure ',num2str(fig1),' is the result shown in Fig. 5 of the paper ']);
disp('"Reduced Communication State Estimation for Control of an Unstable Networked Control System".');
disp(' ');


% % Figure 2: all elements of the estimator variance \check{P}(k) 
% % (for completeness, figure is very crowded).
% figure(2);
% for i=1:n
%     for j=1:n
%         subplot(n,n,(i-1)*n+j);
%         plot(squeeze(P(i,j,:)));
%         grid;
%     end;
% end;
